#include <SDL/SDL.h>
#include <stdlib.h>
#include <time.h>
//#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#define MAX_BACKLOG 139
#define BUFLEN 275
#define PORT 1024
#define BOUNCE 1
#define FULLGRAY 0

SDL_Surface* screen;
SDL_Rect* rects;

void printArray(unsigned char* array)
{
	int i;
	for(i=0; i<BUFLEN; i++)
	{
		if (array[i]){
			printf("1");
		}else{
			printf("0");
		}
		
		if((0 == (i+1)%25) && (i)){
			printf("\n");
		}
	}
	printf("\n");
}

void setPixel(unsigned char* array, int x, int y)
{
	int pos=25*y+x;
	array[pos] = 255;
}

//converts array to SDL pixels
void extract(unsigned char* array)
{
	printf("extract\n");
	printArray(array);
	int i,dim;
	for(i=0;i<275;i++)
	{
		dim = array[i];
		SDL_Rect r=rects[i];
		#if FULLGRAY
		SDL_FillRect(screen,&r,(dim<<16)|(dim<<8)|(dim));
		#else
		dim &= (7<<5);
		SDL_FillRect(screen,&r,(dim<<16)|(dim<<8)|(dim));
		#endif
	}
	SDL_Flip(screen);
}

int main(int argc, char *argv[])
{    
	if (argc != 2) {
	fprintf(stderr,"Usage: screen <TCP-Port>\n");
	exit(1);
	}
    
	SDL_Init(SDL_INIT_EVERYTHING);
	screen = SDL_SetVideoMode(500,220,32,SDL_SWSURFACE);
	SDL_FillRect(screen,NULL,0);
	rects=malloc(sizeof(SDL_Rect)*275);
	int i;
	SDL_Rect r;
	for(i=0; i<275; i++)
	{
		r=rects[i];
		r.w=20;
		r.h=20;
		r.x=(i%25)*20;
		r.y=(i/25)*20;
		rects[i]=r;
		//printf("i: %i x: %i y: %i\n",i,r.x,r.y);
		//SDL_FillRect(screen,&r,0xffffff);
	}
	SDL_Event event;

	int sockfd;			//tcp socket descriptor			
	int port=argc>1?atoi(argv[1]):PORT;//tcp port		
	struct sockaddr_in addr;	//this tcp adress
	socklen_t addr_len;		//length of tcp adress

	fd_set rfds;			//set for sockets at which data is expected
	struct timeval timeout;

	int numbytes=0;			//number of successfully read bytes
	unsigned char* buffer=calloc(sizeof(unsigned char)*BUFLEN,1);	//(temporary) read-in buffer
	//extract(buffer);

	//create tcp socket
	sockfd=socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
	if (sockfd == -1)
	{
		printf("could not create tcp-socket\n");
		exit(1);
	}
	//set up tcp adress
	addr.sin_family = AF_INET;
	addr.sin_port=htons(port);
	addr.sin_addr.s_addr=htonl(INADDR_ANY); //get messages from any adress
	memset(addr.sin_zero, '\0', sizeof(addr.sin_zero)); //reset sin_zero to 0
	addr_len=sizeof(addr);

	//bind socket to adress
	if (bind(sockfd, (struct sockaddr*) &addr, addr_len) == -1)
	{
		printf("could not bind tcp-socket\n");
		close(sockfd);
		exit(1);
	}

	//set tcp-port to listen
	if (listen(sockfd, MAX_BACKLOG) == -1)
	{
		printf("could not listen @ tcp-socket\n");
		close(sockfd);
		exit(1);
	}

	printf("Starting server on IP %s:%i\n",inet_ntoa(addr.sin_addr),addr.sin_port);
	//infinite loop to receive messages
	while(1)
	{
		//reset read-set
		FD_ZERO(&rfds);
		//after selecting, socket has to be put back in set
		FD_SET(sockfd, &rfds);

		//wait for action
		timeout.tv_sec=0;	//set new timeval for each loop
		timeout.tv_usec=100000;
		select(sockfd+1, &rfds, NULL, NULL, &timeout);

		while(SDL_PollEvent(&event))
		{
			//printf("%i\n",event.type);
			if((event.type==SDL_MOUSEBUTTONDOWN) || (event.type==12))
			{
				SDL_Quit();
				exit(0);
			}	
		}
		
		if(BOUNCE)
		{
			for (i=0;i<BUFLEN;i++)
				buffer[i]=0;
			//trampoline
			setPixel(buffer,8,9);
			setPixel(buffer,9,9);
			setPixel(buffer,10,9);
			setPixel(buffer,11,9);
			setPixel(buffer,12,9);
			setPixel(buffer,13,9);
			setPixel(buffer,14,9);
			setPixel(buffer,8,10);
			setPixel(buffer,14,10);
			if(numbytes==0)
			{
				setPixel(buffer,10,10);
				setPixel(buffer,11,10);
				setPixel(buffer,12,10);
			}
			//guy
			i=numbytes>=0?4-numbytes:4+numbytes;
			setPixel(buffer,11,i);
			setPixel(buffer,10,i+1);
			setPixel(buffer,12,i+1);
			setPixel(buffer,11,i+2);
			setPixel(buffer,10,i+3);
			setPixel(buffer,11,i+3);
			setPixel(buffer,12,i+3);
			setPixel(buffer,11,i+4);
			setPixel(buffer,10,i+5);
			setPixel(buffer,12,i+5);
			if(numbytes<4)
				numbytes++;
			else
				numbytes=-3;
			extract(buffer);
		}

		if(FD_ISSET(sockfd, &rfds))
		{
			//something at tcp-port
	
			//create socket for client
			struct sockaddr_in c_addr;
			int c_addr_len=sizeof(c_addr);
	
			int new_sockfd=accept(sockfd, (struct sockaddr*) &(c_addr), (socklen_t*) &c_addr_len);
			printf("New client %s:%i on socket %i\n",inet_ntoa(c_addr.sin_addr),c_addr.sin_port,new_sockfd);
			//read from buffer as long as there is input
			while((numbytes=recv(new_sockfd, buffer, BUFLEN,0)) > 0)
			{
				//printf("%i bytes from %s per TCP\n", numbytes, inet_ntoa(c_addr.sin_addr));
				//printArray(buffer);
		    		extract(buffer);
		    		/*while(SDL_PollEvent(&event))
				{
					if((event.type==SDL_MOUSEBUTTONDOWN) || (event.type==12))
					{
						SDL_Quit();
						exit(0);
					}
					if(event.type==SDL_VIDEOEXPOSE)
						extract(buffer);
				}*/
			}

		//close socket when everything is read from buffer
	    	printf("Socket %i hung up \n", new_sockfd);
		close(new_sockfd);
		numbytes=0;	
		}
	}
	return 0;
}
